package com.oxygenxml.validate.nvdl;

import junit.framework.TestCase;

public class NamespaceSpecificationTest extends TestCase {
  
  
  /**
   * Tests for the covers method.
   *
   */
  public void testCovers() {
    doCovers("*");
    doCovers("\\");
    doCovers("#");
  }
  
  /**
   * Test covers with different wildcards.
   * @param w The wildcard.
   */
  public void doCovers(String w) {
    // any namespace
    NamespaceSpecification nss = 
      new NamespaceSpecification(
          NamespaceSpecification.ANY_NAMESPACE, 
          NamespaceSpecification.DEFAULT_WILDCARD);
    assertTrue(nss.covers(NamespaceSpecification.ANY_NAMESPACE));
    assertFalse(nss.covers("otherNamespace"));
    
    // specified namespace, no wildcards.
    nss = new NamespaceSpecification("http://www.oxygenxml.com/test",w);
    assertFalse(nss.covers(NamespaceSpecification.ANY_NAMESPACE));
    assertTrue(nss.covers("http://www.oxygenxml.com/test"));
    assertFalse(nss.covers("http://www.oxygenxml.com/test1"));
    
    // one wildcard at the end.
    nss = new NamespaceSpecification("http://www.oxygenxml.com/test" + w, w);
    assertFalse(nss.covers(NamespaceSpecification.ANY_NAMESPACE));
    assertTrue(nss.covers("http://www.oxygenxml.com/test"));
    assertTrue(nss.covers("http://www.oxygenxml.com/test/test"));
    assertFalse(nss.covers("http://www.oxygenxml.com/xtest"));
    
    // one wildcard in the middle.
    nss = new NamespaceSpecification("http://www.oxygenxml.com/" + w + "test",w);    
    assertFalse(nss.covers(NamespaceSpecification.ANY_NAMESPACE));
    assertTrue(nss.covers("http://www.oxygenxml.com/test"));
    assertTrue(nss.covers("http://www.oxygenxml.com/test/test"));
    assertTrue(nss.covers("http://www.oxygenxml.com/xtest"));
    assertFalse(nss.covers("http://www.oxygenxml.com/testx"));    
    
    // one wildcard at the beginning
    nss = new NamespaceSpecification(w + "http://www.oxygenxml.com/test", w);
    assertFalse(nss.covers(NamespaceSpecification.ANY_NAMESPACE));
    assertTrue(nss.covers("http://www.oxygenxml.com/test"));
    assertFalse(nss.covers("http://www.oxygenxml.com/test/test"));
    assertTrue(nss.covers("xhttp://www.oxygenxml.com/test"));
    assertFalse(nss.covers("xhttp://www.oxygenxml.com/testy"));
    
    // two wildcards
    nss = new NamespaceSpecification(w + "http://www.oxygenxml.com/test" + w, w);
    assertFalse(nss.covers(NamespaceSpecification.ANY_NAMESPACE));
    assertTrue(nss.covers("http://www.oxygenxml.com/test"));
    assertTrue(nss.covers("XXXhttp://www.oxygenxml.com/testXXX"));
    assertTrue(nss.covers("http://www.oxygenxml.com/testXXX"));
    assertTrue(nss.covers("XXXhttp://www.oxygenxml.com/test"));
    assertFalse(nss.covers("http://www.oxygenxmlX.com/test"));
    
    nss = new NamespaceSpecification("http://www.oxygenxml.com/" + w + "test" + w, w);
    assertFalse(nss.covers(NamespaceSpecification.ANY_NAMESPACE));
    assertTrue(nss.covers("http://www.oxygenxml.com/XXXtest"));
    assertTrue(nss.covers("http://www.oxygenxml.com/testXXX"));
    assertTrue(nss.covers("http://www.oxygenxml.com/XXXtestXXXXX"));
    assertFalse(nss.covers("http://www.oxygenxml.comX/XXXtestXXXXX"));
    
    nss = new NamespaceSpecification("http://www.oxygenxml."+ w +"/" + w + "test", w);
    assertFalse(nss.covers(NamespaceSpecification.ANY_NAMESPACE));
    assertTrue(nss.covers("http://www.oxygenxml.com/XXXtest"));
    assertTrue(nss.covers("http://www.oxygenxml.XXX/XXXtest"));
    assertFalse(nss.covers("http://www.oxygenxml.com/XXXtestX"));
    
    // no wildcard, spefified
    nss = new NamespaceSpecification("http://www.oxygenxml."+ w +"/" + w + "test", "");
    assertTrue(nss.covers("http://www.oxygenxml." + w + "/" + w + "test"));
    assertFalse(nss.covers("http://www.oxygenxml." + "XXX" + "/" + w + "test"));
  }

  /**
   * Test for the compete method.
   */
  public void testCompete() {
    doCompete("*", "*");
    doCompete("#", "#");
    doCompete("\\", "\\");
    doCompete("*", "#");
    doCompete("*", "\\");
    doCompete("#", "\\");
  }

  /**
   * Test compete with different wildcards.
   * @param w1 First namespace specification wildcard.
   * @param w2 Second namespace specification wildcard.
   */
  private void doCompete(String w1, String w2) {
    NamespaceSpecification nss1 = new NamespaceSpecification("", w1);
    NamespaceSpecification nss2 = new NamespaceSpecification("", w2);    
    assertTrue(nss1.compete(nss2));
    
    nss1 = new NamespaceSpecification("http://www.oxygenxml.com", w1);
    nss2 = new NamespaceSpecification("http://www.oxygenxml.com", w2);    
    assertTrue(nss1.compete(nss2));

    nss1 = new NamespaceSpecification("http://www.oxygenxml.com/test", w1);
    nss2 = new NamespaceSpecification("http://www.oxygenxml.com", w2);    
    assertFalse(nss1.compete(nss2));

    nss1 = new NamespaceSpecification("http://www.oxygenxml.com" + w1, w1);
    nss2 = new NamespaceSpecification("http://www.oxygenxml.com", w2);    
    assertTrue(nss1.compete(nss2));

    nss1 = new NamespaceSpecification("http://www.oxygenxml.com" + w1, w1);
    nss2 = new NamespaceSpecification("http://www.oxygenxml.com/test", w2);    
    assertTrue(nss1.compete(nss2));
    
    nss1 = new NamespaceSpecification("http://www.oxygenxml2.com" + w1, w1);
    nss2 = new NamespaceSpecification("http://www.oxygenxml.com", w2);    
    assertFalse(nss1.compete(nss2));
    
    nss1 = new NamespaceSpecification("http://www." + w1 + ".com", w1);
    nss2 = new NamespaceSpecification("http://www.oxygenxml." + w2, w2);    
    assertTrue(nss1.compete(nss2));
    
    nss1 = new NamespaceSpecification("http" + w1 + "://www." + w1 + ".com", w1);
    nss2 = new NamespaceSpecification("http://www.oxyg" + w2 + "enxml." + w2, w2);    
    assertTrue(nss1.compete(nss2));
    
    nss1 = new NamespaceSpecification("http://www." + w1 + ".com", w1);
    nss2 = new NamespaceSpecification(
        NamespaceSpecification.ANY_NAMESPACE, 
        NamespaceSpecification.DEFAULT_WILDCARD);    
    assertFalse(nss1.compete(nss2));
    
    nss1 = new NamespaceSpecification(
        NamespaceSpecification.ANY_NAMESPACE, 
        NamespaceSpecification.DEFAULT_WILDCARD);
    nss2 = new NamespaceSpecification(
        NamespaceSpecification.ANY_NAMESPACE, 
        NamespaceSpecification.DEFAULT_WILDCARD);    
    assertTrue(nss1.compete(nss2));
  
    nss1 = new NamespaceSpecification("http://www.oxygenxml.com/test", "");
    nss2 = new NamespaceSpecification("http://www.oxyg" + w2 + "enxml." + w2, w2);    
    assertTrue(nss1.compete(nss2));
    
    nss1 = new NamespaceSpecification("http://www.oxygenxml.com/test", "");
    nss2 = new NamespaceSpecification("http://www.oxygenxml.com/test", "");    
    assertTrue(nss1.compete(nss2));
    
    nss1 = new NamespaceSpecification("http://www.oxygenxml.com/test", "");
    nss2 = new NamespaceSpecification("http://www.oxygenxml.com/test/x", "");    
    assertFalse(nss1.compete(nss2));
  }
}
